/*
 * Cheops Next Generation
 * 
 * Mark Spencer <markster@marko.net>
 *
 * Copyright(C) 1999, Adtran, Inc.
 * 
 * Distributed under the terms of the GNU General Public License (GPL) Version 2
 *
 * General event management definitions
 *
 */

#ifndef _EVENT_H
#define _EVENT_H

#include "cheops-types.h"

typedef struct event_header {
	u32 len;		/* Length of event (including header) */
	u16 hlen;		/* Length of header and options (like proxy) */
	u16 type;		/* Type of event (see below) */
	u32 flags;		/* Event specific flags */
} event_hdr;

/* Max number of events */
#define MAX_EVENT 256

/*
 * Flags for use in all event headers
 */
#define FLAG_FORCE		1	<<  0		/* Do not use cached response */
#define FLAG_PROXY		1	<<	1		/* Use specified proxy for request or
										   specify that reply is coming from
										   given proxy */
#define FLAG_IGNORE		1	<<  2		/* Ignore message if unknown */

/* Event may be followed by additional information like a proxy
   host, and then one of the following bits of information. */

typedef struct event_proxy {
#define HDRTYPE_PROXY	1 << 0
	u8	hdrtype;	/* Type of header option */
	u8  family; 	/* Family (e.g. AF_INET) */
	u16 len;		/* Length of address */
	/* Address data */
} proxy;

/* ============  Standard Event Requests ================= */

/* 
 * Discover a given network, given a start and end,
 * IP, both in network byte order.
 */ 
typedef struct dipv4_event {
#define EVENT_DIPV4 	1
	u32 start;	/* Start address */
	u32 end;	/* End address */
} dipv4_e;


/* ============  Standard Event Replies ================= */

/*
 * Notification that a given IP address has
 * been discovered
 */
typedef struct dipv4_reply {
#define REPLY_DIPV4 	2
	u32 ipaddr;
} dipv4_r;

/*
 * Report an error string and code.
 * the code # is standard, and the 
 * string is some meaningful message.
 *
 * See cerror.h for more.
 */
typedef struct error_reply {
#define REPLY_ERROR 	3
	u16 error;
	u16 len;
} error_r;

typedef union {
		dipv4_e dipv4_e;
		dipv4_r dipv4_r;
		error_r error_r;
} event;


/* An agent is someone who communicates to us over a fd of some
   sort.   */

#define AGENT_TYPE_LOCAL 1
#define AGENT_TYPE_IPV4 2

struct agent_struct;

typedef struct agent_struct agent;

struct agent_struct {
	agent *next;
	int type;	/* Kind of agent */
	int *id;		/* I/O ID */
	int s;		/* Socket */
	void *more; /* Anything else */
};

/*
 * A Cheops event handler takes it, a header, an event, and the source 
 * agent, and handles it.
 */

typedef int (*cheops_event_handler)(event_hdr *h, event *e, agent *a);
#define CHEOPS_EVENT_HANDLER(a) ((cheops_event_handler)(a))

/*
 * Request an agent to a specific remote host.
 */
extern agent *event_request_agent(int agent_type, void *addr, int is_default);



/*
 * Build a local listening agent.  Note that agent_type can be more
 * than one agent type, OR'd together.
 */
extern int event_create_agent(int agent_type);

/*
 * Send an event to an agent
 */
extern int event_send(agent *a, event_hdr *h);

/*
 * Destroy agent
 */
extern void event_destroy_agent(agent *a);

/*
 * Register a handler for an event type
 */

extern int event_register_handler(short type, cheops_event_handler h, int replace);

#endif
